
(defun dired-tar-marked-files (tar-file &optional arg)
  "Run `tar' on marked (or next ARG) files, creating TAR-FILE."
  (interactive "FMake tar archive in file: \nP")
  (let ((output (get-buffer-create (generate-new-buffer " *tar output*")))
        (files (dired-get-marked-files t (or arg (prefix-numeric-value arg)))))
    (message "Making tar archive...")
    (apply #'call-process "tar" nil output nil "-cf" tar-file
           files)
    (message "Making tar archive...done")
;    (with-current-buffer output
      (save-excursion
        (set-buffer output)
      (if (= (buffer-size) 0)
          (kill-buffer nil)
        (goto-char (point-min))
        (insert (format "Running `tar' with arguments %S %S "
                        "-cf" tar-file)
                (mapconcat #'identity files " ")
                " produced the following output:\n\n")
        (pop-to-buffer output)))))

(define-key dired-mode-map "T" 'dired-tar-marked-files)


(defun gzip-filename (file)
  (if (string-match ".gz" file)
      (substring file 0 (string-match ".gz" file))
    file))

(defun dired-add-file-to-tar (tar-file)
  "Add the marked files to the specified tar file.
Will uncompress files to be included in the archive.
Has a hack to allow gzipped tar files.
TJ"
  (interactive 
   (list (read-file-name "Add marked files to: " (dired-current-directory) "" t)))

  (let ((unzip-file-name (if (string-match ".gz" tar-file)
                             (substring tar-file 0 (string-match ".gz" tar-file))
                           nil))
        (marked-files (dired-get-marked-files)))

    (if unzip-file-name
        (shell-command (concat "gunzip " tar-file)))
    
    (mapcar (lambda (file)
              (if (string-match ".gz" file)
                  (shell-command (concat "gunzip " file))))
            marked-files)

    (shell-command
     (concat "tar -rf "
             (if unzip-file-name unzip-file-name tar-file)
             " "
             (mapconcat 
              (lambda (filename)
                (gzip-filename (file-name-nondirectory filename)))
              (dired-get-marked-files)
              " ")))
    
    (if unzip-file-name
        (shell-command (concat "gzip " unzip-file-name)))
    (mapcar (lambda (file)
              (if (string-match ".gz" file)
                  (shell-command (concat "gzip " (substring file 0 -3)))))
            marked-files))
  (revert-buffer))

