;; create a buffer with the function keys listed on it.

;; Author: Tom Wurgler <twurgler@goodyear.com>
;; Created: 11/6/1996
;; Keywords: function, keybinding

;; This file is not part of GNU Emacs.

;; This program is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.
;;
;; A copy of the GNU General Public License can be obtained from
;; the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
;; 02139, USA.
;;

;;; Commentary:

;; This code pops up a buffer showing what the function keys are bound to.
;; The display is active (if you change a binding during an emacs session,
;; keybindings will still display correctly).  After the pop-up, any key
;; returns to the previous window configuration.  You can display keybindings
;; for plain function keys, shift-function keys, control-function keys or
;; control-shift function keys.  This is just a handy way to scope out
;; what you had the keys bound to.

;
; $Id: keys.el,v 1.2 1999-10-01 10:37:33-04 wurgler Exp wurgler $
;
; $Log: keys.el,v $
; Revision 1.2  1999-10-01 10:37:33-04  wurgler
; added control-shift level of key display
;
; Revision 1.1  1996-11-06 09:57:14-05  wurgler
; Initial revision
;
;; add this to .emacs (with the keybinding of your choice):

;; (define-key global-map [kp-add] 'keybindings)
;; (define-key global-map [C-kp-add] 'c-keybindings)
;; (define-key global-map [S-kp-add] 's-keybindings)
;; (define-key global-map [C-S-kp-add] 'c-s-keybindings)
;; (autoload 'keybindings "keys" "Display function key bindings" t)
;; (autoload 'c-keybindings "keys" "Display control function key bindings" t)
;; (autoload 's-keybindings "keys" "Display shift function key bindings" t)
;; (autoload 'c-s-keybindings "keys" "Display control shift function key bindings" t)

;;;###autoload
(defun c-s-keybindings ()
  "Create a buffer containing the current control-shift function key bindings."
  (interactive)
  (keybindings nil nil t))

;;;###autoload
(defun c-keybindings ()
  "Create a buffer containing the current control function key bindings."
  (interactive)
  (keybindings t))

;;;###autoload
(defun s-keybindings ()
  "Create a buffer containing the current shift function key bindings."
  (interactive)
  (keybindings nil t))

;;;###autoload
(defun keybindings (&optional control shift control-shift)
  "Create a buffer containing the current function key bindings."
  (interactive)
  (save-window-excursion
    (save-excursion
      (let ((beg)
            (end)
            (maxc)
            (howmany 15)
            (key-descr))
        (if control
            (setq key-descr
                  (concat
                   (describe-key-briefly [(control f1)]) "\n"
                   (describe-key-briefly [(control f2)]) "\n"
                   (describe-key-briefly [(control f3)]) "\n"
                   (describe-key-briefly [(control f4)]) "\n"
                   (describe-key-briefly [(control f5)]) "\n"
                   (describe-key-briefly [(control f6)]) "\n"
                   (describe-key-briefly [(control f7)]) "\n"
                   (describe-key-briefly [(control f8)]) "\n"
                   (describe-key-briefly [(control f9)]) "\n"
                   (describe-key-briefly [(control f10)]) "\n"
                   (describe-key-briefly [(control f11)]) "\n"
                   (describe-key-briefly [(control f12)]) "\n"
                   (describe-key-briefly [(control print)]) "\n"
                   (describe-key-briefly [(control key-20)]) "\n"
                   (describe-key-briefly [(control pause)]) "\n"))
          (if shift
              (setq key-descr
                    (concat
                     (describe-key-briefly [(shift f1)]) "\n"
                     (describe-key-briefly [(shift f2)]) "\n"
                     (describe-key-briefly [(shift f3)]) "\n"
                     (describe-key-briefly [(shift f4)]) "\n"
                     (describe-key-briefly [(shift f5)]) "\n"
                     (describe-key-briefly [(shift f6)]) "\n"
                     (describe-key-briefly [(shift f7)]) "\n"
                     (describe-key-briefly [(shift f8)]) "\n"
                     (describe-key-briefly [(shift f9)]) "\n"
                     (describe-key-briefly [(shift f10)]) "\n"
                     (describe-key-briefly [(shift f11)]) "\n"
                     (describe-key-briefly [(shift f12)]) "\n"
                     (describe-key-briefly [(shift print)]) "\n"
                     (describe-key-briefly [(shift key-20)]) "\n"
                     (describe-key-briefly [(shift pause)]) "\n"))
            (if control-shift
                (setq key-descr
                      (concat
                       (describe-key-briefly [(control shift f1)]) "\n"
                       (describe-key-briefly [(control shift f2)]) "\n"
                       (describe-key-briefly [(control shift f3)]) "\n"
                       (describe-key-briefly [(control shift f4)]) "\n"
                       (describe-key-briefly [(control shift f5)]) "\n"
                       (describe-key-briefly [(control shift f6)]) "\n"
                       (describe-key-briefly [(control shift f7)]) "\n"
                       (describe-key-briefly [(control shift f8)]) "\n"
                       (describe-key-briefly [(control shift f9)]) "\n"
                       (describe-key-briefly [(control shift f10)]) "\n"
                       (describe-key-briefly [(control shift f11)]) "\n"
                       (describe-key-briefly [(control shift f12)]) "\n"
                       (describe-key-briefly [(control shift print)]) "\n"
                       (describe-key-briefly [(control shift key-20)]) "\n"
                       (describe-key-briefly [(control shift pause)]) "\n"))
              (setq key-descr
                    (concat
                     (describe-key-briefly [(f1)]) "\n"
                     (describe-key-briefly [(f2)]) "\n"
                     (describe-key-briefly [(f3)]) "\n"
                     (describe-key-briefly [(f4)]) "\n"
                     (describe-key-briefly [(f5)]) "\n"
                     (describe-key-briefly [(f6)]) "\n"
                     (describe-key-briefly [(f7)]) "\n"
                     (describe-key-briefly [(f8)]) "\n"
                     (describe-key-briefly [(f9)]) "\n"
                     (describe-key-briefly [(f10)]) "\n"
                     (describe-key-briefly [(f11)]) "\n"
                     (describe-key-briefly [(f12)]) "\n"
                     (describe-key-briefly [(print)]) "\n"
                     (describe-key-briefly [(key-20)]) "\n"
                     (describe-key-briefly [(pause)]) "\n")))))
        (message nil)

        (if (one-window-p)
            (split-window-vertically))
        (set-buffer (get-buffer-create " *Keybindings*"))
        (setq buffer-read-only nil)
        (erase-buffer)
        (insert key-descr)

        (goto-char 1)
        ;(replace-string " runs the command" "")
        (while (search-forward " runs the command" nil t)
          (replace-match "" nil t))
        (goto-char 1)
        (setq maxc 0)
        (setq count 0)
        (while (not (= count howmany))
          (setq count (1+ count))
          (end-of-line)
          (if (> (current-column) maxc)
              (setq maxc (current-column)))
          (forward-line 1))
        (setq maxc (1+ maxc))
        (goto-char 1)
        (setq count 0)
        (while (not (= count howmany))
          (setq count (1+ count))
          (end-of-line)
          (while (< (current-column) maxc)
            (insert " "))
          (forward-line 1))
        (setq beg (point))
        (setq maxc 0)
        (setq count 0)
        (while (not (= count howmany))
          (setq count (1+ count))
          (end-of-line)
          (if (> (current-column) maxc)
              (setq maxc (current-column)))
          (forward-line 1))
        (setq maxc (1+ maxc))
        (goto-char beg)
        (setq count 0)
        (while (not (= count howmany))
          (setq count (1+ count))
          (end-of-line)
          (while (< (current-column) maxc)
            (insert " ")
            (setq end (point)))
          (forward-line 1))
        (setq end (1- (point)))
        (kill-rectangle beg end)
        (goto-char 1)
        (end-of-line)
        (yank-rectangle)
        (goto-char (1+ end))
        (setq beg (point))
        (setq maxc 0)
        (while (not (eobp))
          (setq count (1+ count))
          (end-of-line)
          (if (> (current-column) maxc)
              (setq maxc (current-column)))
          (forward-line 1))
        (forward-line -1)
        (end-of-line)
        (while (< (current-column) maxc)
          (insert " "))
        (setq end (point))
        (kill-rectangle beg end)
        (goto-char 1)
        (end-of-line)
        (yank-rectangle)
        (setq h (1- (window-height)))
        (pop-to-buffer " *Keybindings*")
        (setq buffer-read-only t)
        (shrink-window (- h (1- howmany)))
        (goto-char 1))
      (setq junkme (read-key-sequence nil)))))

(provide 'keys)