;; Default Emacs Initialisation file
;; Date 15.08.2003
;; MiFi
(setq load-path (append (list             "/usr/share/emacs/site-lisp/auctex"
                                          "/usr/share/emacs/site-lisp/reftex"
                                          "~/local-lisp") load-path ))
(add-to-list 'load-path "/usr/share/emacs/site-lisp/ps-print/lisp/ps-print")

;; Redefine some keys
(global-set-key [home] 'beginning-of-line)
(global-set-key [?\M-s] 'isearch-forward-regexp)
(global-set-key [end] 'end-of-line)
(global-set-key [C-home] 'beginning-of-buffer)
(global-set-key [C-end] 'end-of-buffer)
(global-set-key [f5] 'delete-other-windows)
(global-set-key [C-f9] 'align)
(global-set-key [C-f10] 'Info-speedbar-browser)
(global-set-key [C-f11] 'speedbar)
(global-set-key [C-f12] 'flyspell-mode)
(global-set-key [C-M-down-mouse-1] 'kill-this-buffer)
(global-set-key "\C-cg" 'goto-line)
;(global-set-key [M-f6] 'goto-line)
(global-set-key "\C-ck" 'calendar)
(global-set-key "\eg" 'fill-region)
(global-set-key [C-up]   'enlarge-window)
(global-set-key [C-down]  'shrink-window)

;;; Provides the command shell-toggle which toggles between the
;;; *shell* buffer and whatever buffer you are editing.
(autoload 'shell-toggle "shell-toggle" 
 "Toggles between the *shell* buffer and whatever buffer you are editing." t)
(autoload 'shell-toggle-cd "shell-toggle" 
 "Pops up a shell-buffer and insert a \"cd <file-dir>\" command." t)
(global-set-key [C-f1] 'shell-toggle)
(global-set-key [C-S-f1] 'shell-toggle-cd)

;;tramp
(require 'tramp)
(setq tramp-default-method "scp")

; I would like to be able to page up and page down without losing my
; original position, ie if I hit PGUP 4 times, then PGDN 4 times my cursor
; should be in the same place as it was when i started.  At the moment,
; PGUP and PGDN don't appear to move a fixed amount with each keypress,
; they seem to change depending on the cursor's position relative to the
; top and bottom of the frame.  Can anyone help with this?
(setq scroll-preserve-screen-position t)


;; frequently visited files:
;.emacs (register e): c-x r j e
(set-register ?e '(file . "~/.emacs"))

;; tlac medzinarodnych textov: treba mat nainstalovany balik intlfonts (iba pre X server)
;(setq bdf-directory-list '("/usr/share/emacs/fonts/bdf/"))
;(setq ps-multibyte-buffer 'bdf-font)

;; nedavaj jednopismenove slova na koniec riadku
;; This code is from Stefan Monnier
(defun fill-french-nobreak-p ()
  (or (looking-at "[ \t]*[])}?!;:-]")
      (save-excursion
	(skip-chars-backward " \t")
	(unless (bolp)
	  (backward-char 1))
	(or (looking-at "[([{]")
	    ;; Don't cut right after a single-letter word.
	    (and (memq (preceding-char) '(?\t ?\ ))
		 (eq (char-syntax (following-char)) ?w))))))

(setq fill-nobreak-predicate 'fill-french-nobreak-p)

(defvar goto-last-change-undo nil
  "The `buffer-undo-list' entry of the previous \\[goto-last-change] command.")
(make-variable-buffer-local 'goto-last-change-undo)

;;;###autoload
(defun goto-last-change (&optional mark-point)
  "Set point to the position of the last change.
Consecutive calls set point to the position of the previous change.
With a prefix arg (optional arg MARK-POINT non-nil), set mark so \
\\[exchange-point-and-mark]
will return point to the current position."
  (interactive "P")
  ;; (or (buffer-modified-p)
  ;;     (error "Buffer not modified"))
  (and (eq buffer-undo-list t)
       (error "No undo information in this buffer"))
  (if mark-point (push-mark))
  (let ((position nil)
	(undo-list (if (and (eq this-command last-command)
			    goto-last-change-undo)
		       (cdr (memq goto-last-change-undo buffer-undo-list))
		     buffer-undo-list))
	undo)
    (while undo-list
      (setq undo (car undo-list))
      (cond ((and (consp undo) (integerp (car undo)) (integerp (cdr undo)))
	     ;; (BEG . END)
	     (setq position (cdr undo)
		   undo-list '()))
	    ((and (consp undo) (stringp (car undo))) ; (TEXT . POSITION)
	     (setq position (abs (cdr undo))
		   undo-list '()))
	    ((and (consp undo) (eq (car undo) t))) ; (t HIGH . LOW)
	    ((and (consp undo) (null (car undo)))
	     ;; (nil PROPERTY VALUE BEG . END)
	     (setq position (cdr (last undo))
		   undo-list '()))
	    ((and (consp undo) (markerp (car undo)))) ; (MARKER . DISTANCE)
	    ((integerp undo))		; POSITION
	    ((null undo))		; nil
	    (t (error "Invalid undo entry: %s" undo)))
      (setq undo-list (cdr undo-list)))
    (cond (position
	   (setq goto-last-change-undo undo)
	   (goto-char position))
	  ((and (eq this-command last-command)
		goto-last-change-undo)
	   (setq goto-last-change-undo nil)
	   (error "No further undo information"))
	  (t
	   (setq goto-last-change-undo nil)
	   (error "Buffer not modified")))))


 (global-set-key "\C-x\C-\\" 'goto-last-change)


;; Kill current buffer silently if it hasn't been altered
(defun kill-current-buffer ()
  "Kill the current buffer (prompting if it is modified)."
  (interactive)
  (kill-buffer (current-buffer)))
(global-set-key '[(control x)(k)] 'kill-current-buffer)



;; This package provides the commands `swbuff-switch-to-next-buffer'
;; and `swbuff-switch-to-previous-buffer' to respectively switch to
;; the next or previous buffer in the buffer list.
;; C-f6, C-S-f6  (MiFi)
(require 'swbuff)

;;here is function that cycles through buffers with the same mode:
(defun cycle-mode-buffer (&optional mode dont-match)
  (interactive)
  (let* ((cm (or mode mode-name))
         next-buf)
    (catch 'done
      (if (not mode)
          (bury-buffer (current-buffer)))
      (let ((bl (buffer-list))) 
        (while bl (let* ((buf (car bl))
                         (bn (buffer-name buf)))
                    (if (and (not (or 
                                   (string-match "\\` " bn)
                                   (and dont-match
                                        (string-match dont-match bn))))
                             (save-excursion (set-buffer buf)
                                             (string-equal cm mode-name)))
                        (throw 'done (switch-to-buffer buf))
                      (setq bl (cdr bl)))))))))
(global-set-key [f7] 'cycle-mode-buffer)


;; nice printing dialogs, 1,2,4 up printing, etc.
(setq ps-postscript-code-directory "/usr/share/emacs/site-lisp/ps-print/etc/ps-print/")
(require 'printing)
(setq ps-bold-faces
      '(font-lock-comment-face
        font-lock-keyword-face
        font-lock-builtin-face
        font-lock-function-name-face))
(setq ps-italic-faces
      '(font-lock-comment-face
        font-lock-variable-name-face
        font-lock-string-face))
(setq ps-underlined-faces
      '(font-lock-type-face
        font-lock-reference-face
        font-lock-builtin-face))


; Die AltGr-Taste soll im Emacs die Bedeutung Alt statt Meta haben.
(setq default-frame-alist (append '((altgr-modifier . alt))
                                  default-frame-alist))

;; Die Grossschreibung bei NFS wird beibehalten
(setq case-fold-file-names nil)
(setq preserve-file-name-case t)

;;; Set up some initial things

;; This package is a minor mode to highlight the current line in buffer.
;(require 'highline)
;; Turn on local highlighting for Dired (C-x d)
;(add-hook 'dired-after-readin-hook 'highline-on)
;; Turn on local highlighting for list-buffers (C-x C-b)
;(defadvice list-buffers (after highlight-line activate)
;  (save-excursion
;    (set-buffer "*Buffer List*")
;    (highline-on)))
;(highline-mode-on)

(cond ((file-exists-p "~/.backups")
       (setq auto-save-list-file-prefix "~/.backups/.saves-")))
(setq c-brace-offset -3)
(setq perl-indent-level 2)
(setq tab-width 2)
(setq tab-interval 2)

;; ; Allow numbered backups
(setq make-backup-files t)
(setq trim-versions-without-asking t)
(setq version-control t) 
(setq delete-old-versions t) 

;; Auto-save
(setq auto-save-interval 50)

(setq next-line-add-newlines nil)
(setq set-fill-column 79)

(setq visible-bell t)
(setq-default indent-tabs-mode nil)

;(modify-syntax-entry "_" "w")
(let ((process-connection-type nil))	; try not to hog a pty for this.
  (display-time))

(setq frame-title-format "Rayserver - Emacs - %b")
(setq icon-title-format "Rayserver - Emacs - %b")

;;; Resize the minibuffer so its entire contents are visible.
(setq resize-minibuffer-mode t)
(resize-minibuffer-mode)

;;; set time display to 24h format
(setq display-time-24hr-format t)

;; ---------------------------------------------------------------------------
;; to get emacs a bit more consistent, replace all yes or no questions with
;; simple y or n. 
;; ---------------------------------------------------------------------------
(fset 'yes-or-no-p 'y-or-n-p)

; *Non-nil means vertical motion starting at end of line keeps to ends of lines.
; This means moving to the end of each line moved onto.
; The beginning of a blank line does not count as the end of a line.
(setq track-eol t)

; Emulate Windows GUI selection keys
; (setq pc-selection-mode t)

;; Version: 2.4
;; This is the CUA-mode package which provides a complete emulation of
;; the standard CUA key bindings (Motif/Windows/Mac GUI) for selecting
;; and manipulating the region where S-<movement> is used to
;; highlight & extend the region.
;; To activate CUA-mode, copy the cua.el file into a directory
;; on the load-path and place the following lines in your .emacs:
;;
(load "cua")
(CUA-mode t)

;;; OS specific stuff
;(set-default-font "-*-Courier-normal-r-*-*-13-97-96-96-c-*-iso8859-1")
;(set-default-font "-*-Fixedsys-normal-r-*-*-15-112-96-96-c-80-iso8859-1")
(global-set-key [mouse-wheel] 'mouse-wheel-handler)
(setq shell-command-switch "-c")
; (setq win32-quote-process-args t)
; (setq win32-num-mouse-buttons 2)

(setq default-frame-alist
      '(
        (cursor-color . "blue")
        (foreground-color . "black")
        (background-color . "linen")
        (top . 1) (left . 1)
        (width . 140) (height . 86)
        ))

;(setq initial-frame-alist '((top . 2) (left . 250)))
;(setq initial-frame-alist '((top . 10) (left . 1) (width . 160) (height . 144)))
(setq initial-frame-alist
      '(
        (top . 1) (left . 1)
        (width . 140) (height . 40)
        ))

; maximise Emacs on startup
;(add-hook 'term-setup-hook 'maximize-frame)

 (add-hook 'shell-mode-hook
           '(lambda () (setq comint-completion-addsuffix '("/" . ""))) t)

;; Colorized fonts
(global-font-lock-mode t)

(setq query-replace-highlight t)	;highlight during query
(setq search-highlight t)		;incremental search highlights


;(setq completion-auto-help t)           ;; I want as much help as I can get
;(setq completion-auto-exit t)           ;; Don't want to always hit return; see what you mark

; do not print grey colours on BW printer
(setq ps-print-color-p nil)

; scroll only one line also out of window limits
(setq scroll-conservatively 1)
(setq scroll-step 1)

(global-set-key "\C-x\C-b" 'buffer-menu)

;**************************************************************
;**************************************************************
;; Dired Mode 
;**************************************************************
;**************************************************************
;; This package, which is intended for use with GNU Emacs 20/21,
;; implements a menu and a dialogue that consist of a set of radio
;; buttons and toggles to control sort order in dired mode.  The menu
;; is added as a submenu to the Immediate menu, and bound to S-mouse-2
;; as a popup menu.  The dialogue can be accessed via the menu or the
;; dired key binding `C-d' or the command `dired-sort-dialogue'
(add-hook 'dired-load-hook
           (lambda ()
             (require 'dired-a)
             (require 'ls-lisp)
             (require 'dired-sort-menu)))
(load "dired-x")
;;; dired-a Various dired extensions.
;;  1. The delete command can delete a non-empty directory by deleting
;;     all its files and subdirectories.
;;  2. The copy command can copy a directory by copying a directory and
;;     all its files and subdirectories.
;;  3. Handle archives (tar, arc, lharc, zoo etc.). This is not a replacement
;;     for tar-mode or archive-mode but it complements them. There are commands
;;     unpacking an entire archive, extracting specified files (it might be
;;     better to use tar-mode etc. for this) and get a content listing.
;;     Furthermore the copy command is extended to copy marked files to a
;;     target that is an archive.
;;  4. View files with a method depending on file name.
;;  5. Print files with a method depending on file name.
;;  6. Remove carriage return at end of lines for marked files.
;;  7. Toggle case for file base name (excluding the extension).

;; only one dired window instead of many
(defadvice dired-advertised-find-file (around dired-subst-directory activate)
  "Replace current buffer if file is a directory."
  (interactive)
  (let* ((orig (current-buffer))
         (filename (dired-get-filename)))
    ad-do-it
    (when (and (file-directory-p filename)
               (not (eq (current-buffer) orig)))
      (kill-buffer orig))))

(defadvice dired-up-directory (around  kill-last-buffer())
  "Kill last dired buffer when upping directories (unless upping is impossible...)"
  (let ((last-dired-buffer (current-buffer)))
	ad-do-it
	(let ((current-dired-buffer (current-buffer)))
	  (if (not (eq current-dired-buffer last-dired-buffer))
		  (kill-buffer last-dired-buffer)))))
(ad-activate 'dired-up-directory)

(define-key dired-mode-map (kbd "<insert>") 'dired-mark)
(define-key dired-mode-map (kbd "<delete>") 'dired-flag-file-deletion)
(define-key dired-mode-map (kbd "S-<delete>") 'mifi-delete-flag-files)

(defun mifi-delete-flag-files ()
  "Flag for deletion all files that begin with exe, or have extension .o or are backups. This is nice for fortran/C rubbish files."
  (interactive)
  (dired-flag-files-regexp "^exe.*\\|\\.o$\\|.*~$\\|TAGS"))
;; delete non-empty directories (v 21)
(setq dired-recursive-deletes t)
dired-recursive-copies
;; copy non-empty directories (v 21)
(setq dired-recursive-copies t)


;;; Does anyone know of an extension (or already built in function) to the
;;; dired-mode to sort files and directories in the same way as the IE does,
;;; i.e. ignore case for directory and filenames.
;;;======================================================================
;;; ls-lisp
;;;======================================================================
;;; load the new ls-lisp file package, which provides a nicer format
;;; for the dired mode listings
;;;(add-hook 'dired-load-hook
;;;      (lambda () (load-library "ls-lisp"))
;;;      (lambda () (require 'dired-sort-menu)))

;;; sort the dired buffer to place the directories on top (t)
(setq ls-lisp-dirs-first t)

;;; sort without regard to case (t)
(setq ls-lisp-ignore-case t)


;;norton commander?
(autoload 'nc "nc" "Emulate MS-DOG file shell" t)



;; Samba drive file system
;(add-untranslated-filesystem "S:")

;; This package provides text based table creation and editing
;; feature.  With this package Emacs is capable of editing tables that
;; are embedded inside a text document, the feature similar to the
;; ones seen in modern WYSIWYG word processors. 
;(require 'table)
;;
;; Have the next expression also, if you want always be ready to edit
;; tables inside text files.  This mechanism is analogous to
;; fontification in a sense that tables are recognized at editing time
;; without having table information saved along with the text itself.
;;
;(add-hook 'text-mode-hook 'table-recognize)


;;; Set up abbreviation mode
(setq-default abbrev-mode t)
(cond ((file-exists-p "~/.abbrev_defs")
       (read-abbrev-file "~/.abbrev_defs")))
(setq save-abbrevs t)

;(defun shell-mode-settings ()
;  (add-hook 'comint-output-filter-functions 
;            'comint-watch-for-password-prompt nil t)
;  (add-hook 'comint-output-filter-functions 
;            'comint-strip-ctrl-m nil t)
;  (setq tab-width 8))

;(add-hook 'shell-mode-hook 'shell-mode-settings)

;;; Setup functions to allow toggling between quoting args and not
;(defun sk-quote-process-args (state)
;  (interactive "sState: ")
;  (cond ((equal state "t")
;         (setq win32-quote-process-args t))
;        (not equal (state "t")
;             (setq win32-quote-process-args t))))

(put 'eval-expression 'disabled nil)


;;; Start off in "home" dir.
;;(cd (getenv "HOME"))

;;
;; Put this file in the emacs load-path so emacs can find it (check the manual).

;; set text-mode as default
(setq default-major-mode 'text-mode)
(add-hook 'text-mode-hook 'turn-on-auto-fill)


; ;; start the gnuserver
; (load "gnuserv")
; (setq gnuserv-frame t)
; (gnuserv-start)

; ;By default, gnuserv will load files into new frames. 
; ;If you would rather have gnuserv load files into an existing frame, 
; ;then evaluate the following in the chosen frame: 
; (setq gnuserv-frame (selected-frame))

; ;; Einstellungen zum drucken 
; (setq-default w32-print-ps-destination "FILE:"
;               w32-print-paper-type 'a4
;               w32-print-psnup-path "c:/texmf/miktex/bin")

; (setq-default w32-print-gs-destination "//kvasnica/hp"
;               w32-print-gs-device "ljet4"
;               w32-print-gs-device-dpi '600
;               w32-print-gs-version "6.01"
;               w32-print-gs-path "c:/gs/gs5.01"
;               w32-print-gs-view-path "c:/gs/gsview"
;               w32-print-use-gs t)
; (load "w32-print" 'noerror 'nomessage)

; start emacsclient, save/finish with C-x #.
; not for Windows - use gnuserver instead
(server-start)
(setq main-frame (selected-frame))
(add-hook 'server-visit-hook '(lambda ()
(raise-frame main-frame)
                                (x-focus-frame main-frame)))


;; Formatierung von Tabellen und Quellcode
(require 'align)


;; Recent Files Menu in Files (MiFi 12.09.00)
(require 'recentf)
(recentf-mode 1) 

; how can I write a macro to close all open buffers
(global-set-key [(\M f10)] 'mifi-clear-all-emacs-buffers)
(defun mifi-clear-all-emacs-buffers ()
  "kill all buffers, leaving *scratch* only"
  (interactive)
  ;;close all frames but one, first
  (mapcar (lambda (x) (delete-frame x))
          (cdr (visible-frame-list)))
  ;;then, kill all buffers
  (mapcar (lambda (x) (kill-buffer x))
          (buffer-list))
  ;;make there be only one window holding buffer *scratch*
  (delete-other-windows)
  (iconify-frame))

;**************************************************************
;**************************************************************
;; LaTeX Mode (Auc-TeX + RefTeX + CDLaTeX +...)
;**************************************************************
;**************************************************************

;; load default lisp file (.el) for improved LaTeX mode
;; all settings are located in tex-site, hence
;**************************************************************
(require 'tex-site)
(require 'textilde)
(setq tex-default-mode 'latex-mode)
(add-to-list 'Info-default-directory-list "/usr/share/emacs/site-lisp/auctex/doc")
; (setq LaTeX-section-hook
;       '(LaTeX-section-heading
;         LaTeX-section-title
;         LaTeX-section-toc
;         LaTeX-section-section
;         LaTeX-section-label))
(add-hook 'LaTeX-mode-hook 'outline-minor-mode)   ; with AUCTeX LaTeX mode
(setq outline-minor-mode-prefix "\C-c\C-o") 
;(defun turn-on-outl-mouse-minor-mode ()
;  (outl-mouse-minor-mode 1))
;(add-hook 'LaTeX-mode-hook 'turn-on-outl-mouse-minor-mode)

;  If you want to make AUC TeX aware of style files and multi-file
;  documents right away, insert the following in your `.emacs' file.
(setq enable-local-variables t)
(setq-default TeX-master nil)
(setq TeX-parse-self t)
(setq TeX-auto-save t)

;;;  REFTEX ;;;
(add-to-list 'Info-default-directory-list "~/emacs/site-lisp/reftex/")
(autoload 'reftex-mode     "reftex" "RefTeX Minor Mode" t)
(autoload 'turn-on-reftex  "reftex" "RefTeX Minor Mode" nil)
(autoload 'reftex-citation "reftex-cite" "Make citation" nil)
(autoload 'reftex-index-phrase-mode "reftex-index" "Phrase mode" t)
(add-hook 'LaTeX-mode-hook 'turn-on-reftex)   ; with AUCTeX LaTeX mode
(add-hook 'latex-mode-hook 'turn-on-reftex)   ; with Emacs latex mode

(setq reftex-enable-partial-scans t)
(setq reftex-save-parse-info t)
(setq reftex-use-multiple-selection-buffers t)
(setq reftex-plug-into-AUCTeX t)
(setq reftex-extra-bindings t)
(setq bib-cite-use-reftex-view-crossref t)

(setq reftex-label-alist
  '(("problem" ?p "p:"  "~\\ref{%s}" nil ("problem"   "prob."))
    ("ttt" ?h "ttt:" "~\\ref{%s}" t   ("theorem" "theor." "th."))))


;; Turning on Font-lock automatically:
;;
;;  If you choose to turn-on font-lock by default using a mode-hook,
;;  there is an order to respect with-respect-to loading font-latex.  
;;  Do either:
;;
(if window-system
    (progn
      (require 'font-latex)
      (add-hook 'latex-mode-hook 'turn-on-font-lock 'append)
      (add-hook 'LaTeX-mode-hook 'turn-on-font-lock 'append)))

;;;;Define functions useful for AUC-TeX
;;If you do not want to use 'C-c C-c' or the pull-down menu "Command"
(defun do-LaTeX ()
  "LaTeX the curent file."
  (interactive)
  (save-some-buffers t)    
  (TeX-command "LaTeX" 'TeX-master-file))
(defun do-CsLaTeX ()
  "CsLaTeX the curent file."
  (interactive)
  (save-some-buffers t)    
  (TeX-command "CsLaTeX" 'TeX-master-file))
(defun do-DVIview ()
   "YAPview the curent file."
   (interactive)
   (TeX-command "View" 'TeX-master-file))
(defun do-DVIPS ()
   "DVIPS the curent file."
   (interactive)
   (TeX-command "File" 'TeX-master-file))
(defun do-PSview ()
   "DVIPS the curent file."
   (interactive)
   (TeX-command "ViewPS" 'TeX-master-file))

;;-----------------------------------------------------------
;; post-load hook : LaTeX-mode

;;
;; LaTeX mode enhancements
;;

(eval-after-load
    "tex-site"
  '(add-hook 'LaTeX-mode-hook
             '(lambda () "" (interactive)
 ;               (require 'preview)
 ;               (setq gs-device "x11alpha")
                (define-key LaTeX-mode-map [C-f1] 'hide-subtree)
                (define-key LaTeX-mode-map [C-f2] 'show-entry)
                (define-key LaTeX-mode-map [C-f3] 'hide-body)
                (define-key LaTeX-mode-map [C-f4] 'show-children)
                (define-key LaTeX-mode-map [(control f5)] 'do-LaTeX)
                (define-key LaTeX-mode-map [(control S-f5)] 'do-CsLaTeX)
                (define-key LaTeX-mode-map [(S-f5)] 'do-DVIview)    
                (define-key LaTeX-mode-map [(control f7)] 'do-DVIPS)
                (define-key LaTeX-mode-map [(control f8)] 'do-PSview)
;                (define-key LaTeX-mode-map [f7]     'TeX-command-master)
;                (define-key LaTeX-mode-map [S-f7]   'TeX-next-error)
                (define-key LaTeX-mode-map [f8]     'TeX-insert-macro)
                (define-key LaTeX-mode-map [S-f8]   'LaTeX-environment)
              )             ; end lambda
   )                        ; end add-hook
)                           ; end eval-after-load

;;


(autoload 'cdlatex-mode "cdlatex" "CDLaTeX Mode" t)
(autoload 'turn-on-cdlatex "cdlatex" "CDLaTeX Mode" nil)
(add-hook 'LaTeX-mode-hook 'turn-on-cdlatex)   ; with AUCTeX LaTeX mode


; start des flyspell modes bei LaTeX Dateien
;(add-hook 'LaTeX-mode-hook 'flyspell-mode)
(setq flyspell-default-dictionary "english")


;; Hilit auch f"ur latex und amstex befehle 
(setq hilit-AmSLaTeX-commands t)
(setq font-lock-maximum-decoration t)

;; This package can be typically used for adding forgotten tildes in TeX
;; sources or adding `&nbsp;' sequences in SGML (e.g. HTML) texts.
(autoload 'tildify-buffer "tildify" t)
(autoload 'tildify-region "tildify" t)

;;settings for bib-cite.el
;**************************
(require 'imenu)
(define-key global-map [S-mouse-3] 'imenu)
(autoload 'turn-on-bib-cite "bib-cite")
(add-hook 'LaTeX-mode-hook 'turn-on-bib-cite)
;(setq bib-dos-or-os2-variable t)	

;; Abbreviations for LaTeX
(define-abbrev-table 'LaTeX-mode-abbrev-table nil)
(add-hook 'LaTeX-mode-hook
   (lambda ()
   (setq local-abbrev-table LaTeX-mode-abbrev-table)))

; ;;; Show matching parens
; (require 'paren)
; ;; stig-paren.el, a parentheses mathing much better than the default paren.el
; ;*************** (for better view of matched parentheses)
; (cond (window-system
;        (require 'stig-paren)
;        ;; XEmacs
;        (global-set-key [(control leftparen)] 'stig-paren-toggle-dingaling-mod)
;         (global-set-key [(control rightparen)] 'stig-paren-toggle-sexp-mode))
;        (t
;         (setq blink-matching-paren t)))
; (defvar paren-ding-unmatched t
;   "* Make noise if the cursor is at an unmatched paren.

; If T, then typing or passing over an unmatched paren will ring the bell.
; If NIL, then the bell will not ring even if an unmatched paren is typed.
; If neither T or NIL, then the bell will not ring when the cursor moves over
; unmatched parens but will ring if one is typed.")

;;; mic-paren.el --- advanced highlighting of matching parenthesises.
;;; Version 3.2 - 2000-06-14
(require 'mic-paren) ; loading
(paren-activate)     ; activating
;;; ;; In LaTeX-mode we want this
(add-hook 'LaTeX-mode-hook
          (function (lambda ()
                      (paren-toggle-matching-quoted-paren 1)
                      (paren-toggle-matching-paired-delimiter 1))))


;**************************************************************
;**************************************************************
;; Matlab Mode
;**************************************************************
;**************************************************************

;; You may wish to add something like the following to your ~/.emacs file:
(autoload 'matlab-mode "matlab" "Enter Matlab mode." t)
(setq auto-mode-alist (cons '("\\.m$" . matlab-mode) auto-mode-alist))
(autoload 'matlab-shell "matlab" "Interactive Matlab mode." t)
(setq matlab-indent-function t)	; if you want function bodies indented
;; (setq matlab-verify-on-save-flag nil) ; turn off auto-verify on save
(defun my-matlab-mode-hook ()
     (turn-on-auto-fill)
     (global-font-lock-mode t));; To get font-lock try adding
;;     (matlab-mode-hilit));; To get hilit19 support try adding
(add-hook 'matlab-mode-hook 'my-matlab-mode-hook)
( setq matlab-shell-command "d:/unix/matlabShell.exe"
       matlab-shell-command-switches "500 10000"
	 shell-command-echoes nil)

(defun my-matlab-shell-mode-hook ()
  '())
(add-hook 'matlab-shell-mode-hook 'my-matlab-shell-mode-hook)

;**************************************************************
;**************************************************************
;;  Compile Mode
;**************************************************************
;**************************************************************
;;;
;;; Compilation command.  Will run contents of variable
;;; mifi-compile-command in compilation buffer If mifi-compile-command
;;; doesn't exist, a proper one is searched for. If GNUmakefile
;;; exists, or if Makefile contains GNU make specific statements,
;;; "gmake" is run, otherwise the value of compile-command is run or
;;; if that doesn't exit "make".  Calling this function with a prefix
;;; causes it to read a command from the minibuffer.
;;;

; (defun compile-goto-next-error ()
;   "Switch to compilations output buffer and get next error"
;   (interactive)
;   (require 'compile)
;   (set-buffer "*compilation*")
;   (compilation-next-error 1)
;   (compile-goto-error)
;   )


;(defvar mifi-compile-command nil)
;(defun initial-mifi-compile-command ()
;  (if (file-exists-p "makefile")
;      "make -f makefile"
;    (if (or (file-exists-p "GNUmakefile.mifi")
;	    (file-exists-p "GNUmakefile")
;	    (eq 0 (call-process "egrep" nil nil nil "^include" "Makefile")))
;	"gmake"
;      "make")))

; (defun initial-mifi-compile-command ()
;   (if (file-exists-p "GNUmakefile.mifi")
;       "gmake -f GNUmakefile.mifi"
;     (if (or (file-exists-p "GNUmakefile.mifi")
; 	    (file-exists-p "GNUmakefile")
; 	    (eq 0 (call-process "egrep" nil nil nil "^include" "Makefile")))
; 	"gmake"
;       "make")))

;(defun mifi-compile () (interactive)
;  (if current-prefix-arg
;      (setq mifi-compile-command
;	    (if (not mifi-compile-command)
;		(read-from-minibuffer "Compile command: "
;				      (initial-mifi-compile-command))
;		(read-from-minibuffer "Compile command: "
;				      mifi-compile-command))))
;  (if (not mifi-compile-command)
;      (setq mifi-compile-command (initial-mifi-compile-command)))
;  (save-some-buffers t)    
;  (compile mifi-compile-command))

(defun compile-current-buffer()
  (interactive)
  (save-some-buffers t)    
  (compile (concat "gcc -Wall -g " (buffer-file-name) " -lm")))

(defvar makefile-name "makefile"
  "Name of the makefile for the current file. Some files may have this
variable defined locally.")
(defvar mifi-compile-command nil
  "MiFi compile command is based on the local bufer variable `makefile-name'")

(defun mifi-compile () (interactive)
  (if current-prefix-arg
      (setq mifi-compile-command
	    (if (not mifi-compile-command)
		(read-from-minibuffer "Compile command: "
				      (initial-mifi-compile-command))
		(read-from-minibuffer "Compile command: "
				      mifi-compile-command))))
  (if (not mifi-compile-command)
      (setq mifi-compile-command (concat "make -f " makefile-name)))
  (save-some-buffers t)    
  (compile mifi-compile-command)
  (setq mifi-compile-command nil))

(defvar mifi-debug-command nil)
(defun initial-mifi-debug-command ()
  (if (file-exists-p "exe")
      "gdb exe"
    (if (or (file-exists-p "for")
	    (file-exists-p "res")
	    (eq 0 (call-process "egrep" nil nil nil "^include" "Makefile")))
	"gdb"
      "gdb")))

(defun mifi-debug () (interactive)
  (if current-prefix-arg
      (setq mifi-debug-command
	    (if (not mifi-debug-command)
		(read-from-minibuffer "Debug command: "
				      (initial-mifi-debug-command))
		(read-from-minibuffer "Debug command: "
				      mifi-debug-command))))
  (if (not mifi-debug-command)
      (setq mifi-debug-command (initial-mifi-debug-command)))
  (gdb mifi-debug-command))


;**************************************************************
;**************************************************************
;;  C/C++ MODE
;**************************************************************
;**************************************************************
;; Setup C/C++/Java indentation
(require 'cc-mode)
; This will work after redumping
;(setq c-site-default-style "local")
;
;;; Use this until redumping
;(add-hook 'c-mode-hook
;	  (function (lambda ()
;		      (setq c-file-style "local"))))
;(add-hook 'c++-mode-hook
;	  (function (lambda ()
;		      (setq c-file-style "local"))))
;
;(c-add-style "local"
;             '((c-basic-offset . 2)
;               (c-comment-only-line-offset . 0)
;               (c-hanging-braces-alist . ((substatement-open after)))
;               (c-offsets-alist . ((topmost-intro        . 0)
;                                   (topmost-intro-cont   . 0)
;                                   (substatement         . 2)
;                                   (substatement-open    . 0)
;                                   (statement-case-open  . 2)
;                                   (statement-cont       . 2)
;                                   (access-label         . -2)
;                                   (inclass              . 2)
;                                   (inline-open          . 2)
;                                   ))))
;(c-add-style "tabs"
;             '((c-basic-offset . 2)
;               (c-comment-only-line-offset . 0)
;               (c-offsets-alist . ((statement-block-intro . +)
;                                  (knr-argdecl-intro . 0)
;                                  (substatement-open . 0)
;                                  (label . -)
;                                  (statement-cont . 0)
;                                  ))))

;;; Go into c++-mode or perl-mode depending on file extention
(setq auto-mode-alist
      (append '(("\\.C$"   . c++-mode)
		("\\.cc$"  . c++-mode)
		("\\.cpp$" . c++-mode)
		("\\.cxx$" . c++-mode)
		("\\.hxx$" . c++-mode)
		("\\.h$"   . c++-mode)
		("\\.hh$"  . c++-mode)
		("\\.idl$" . c++-mode)
		("\\.c$"   . c-mode)
                ("\\.pl$" . perl-mode)
                ("[mM]ak\\(efile\\)?$"	. makefile-mode)
                ("\\.pm$" . perl-mode)
                ("\\.java$" . java-mode)
                ("\\.?[Ff][Aa][Qq]$" . faq-mode)
                ("\\.txt$" . text-mode))
	      auto-mode-alist))


; ;; Here's a sample .emacs file that might help you along the way.  Just
; ;; copy this region and paste it into your .emacs file.  You may want to
; ;; change some of the actual values.

; (defconst my-c-style
;   '((c-tab-always-indent        . t)
;     (c-comment-only-line-offset . 2)
;     (c-hanging-braces-alist     . ((substatement-open after)
;                                    (brace-list-open)))
;     (c-hanging-colons-alist     . ((member-init-intro before)
;                                    (inher-intro)
;                                    (case-label after)
;                                    (label after)
;                                    (access-label after)))
;     (c-cleanup-list             . (scope-operator
;                                    empty-defun-braces
;                                    defun-close-semi))
;     (c-offsets-alist            . ((arglist-close . c-lineup-arglist)
;                                    (substatement-open . 0)
;                                    (case-label        . 2)
;                                    (block-open        . 0)
;                                    (knr-argdecl-intro . -)))
;     (c-echo-syntactic-information-p . t)
;     )
;   "My C Programming Style")

; ;; offset customizations not in my-c-style
; (setq c-offsets-alist '((member-init-intro . ++)))

; ;; Customizations for all modes in CC Mode.
; (defun my-c-mode-common-hook ()
;   ;; add my personal style and set it for the current buffer
;   (c-add-style "PERSONAL" my-c-style t)
;   ;; other customizations
;   (setq tab-width 8
;         ;; this will make sure spaces are used instead of tabs
;         indent-tabs-mode nil)
;   ;; we like auto-newline and hungry-delete
;   (c-toggle-auto-hungry-state 1)
;   ;; keybindings for all supported languages.  We can put these in
;   ;; c-mode-base-map because c-mode-map, c++-mode-map, objc-mode-map,
;   ;; java-mode-map, idl-mode-map, and pike-mode-map inherit from it.
;   (define-key c-mode-base-map "\C-m" 'c-context-line-break)
;   )

; (add-hook 'c-mode-common-hook 'my-c-mode-common-hook)


;;{{{ post-load hook : cc-mode

;;----------------------------------------------------------------------------------------
;; C/C++/Java/IDL/etc common configuration hook
;; ---------------------------------------------------------------------
;; Put it into the c-mode-common-hook so that these items are set
;; properly whenever a cc-mode derivative is setup by emacs.
;; ---------------------------------------------------------------------
;; Clears local key mappings for ^TAB, ^S, ^Xb, A-S and ^X-^B.
;; Sets local key mappings for the following:
;;  Alt-;      - inserts a c-scope-operator (the ::)
;;  return     - remapped to \"newline-and-indent\" for electric
;;               autoformatting in code
;;  f5         - shell-command
;;  C-f5         - compile / recompile
;;  ... and if we're running emacs20 the following are also created ...
;;  ESC-r      - query-replace
;;  ESC-Ctrl-r - query-replace-regexp
;;  ESC-Ctrl-Z - save-some-buffers-quiet
;; ---------------------------------------------------------------------
;; Additinally, and more significantly, configures a coding styles based
;; upon the basic \"bsd\" style that has the following additional
;; attributes:
;; - electric-characters: ;, {, and #
;; - the backspace key is set to \"hungry\" mode
;; - the comment continuation stars are set to \"* \"
;; - the block-comment closure \"*/\" will be put onto a line by itself
;;   during auto-format
;; - the comment column is set to 96
;; - the fill column is set to 96
;; - the tab width is set to 2
;; - a comment-only line entered outside any function body will indent
;;   four columns to start with.
;; - Alt-: will insert a C++ scope operator for you
;; - Enables the following cleanups to be done:
;;    empty-defun-braces   defun-close-semi
;;    list-close-comma     scope-operator
;;

;;
;; c/c++ mode syntax table modes, do this once on module load
;;

(eval-after-load
    "cc-mode"
  '(progn (modify-syntax-entry ?_ "w" c-mode-syntax-table)
          (modify-syntax-entry ?_ "w" c++-mode-syntax-table)
;           (modify-syntax-entry ?\> ")<" c++-mode-syntax-table)
;           (modify-syntax-entry ?\< "(>" c++-mode-syntax-table)
  
                 (c-add-style
                  "my-cc-style"
                  '((indent-tabs-mode . nil) ;no TABs when nil
                    (comment-column . 40)
                    (c-auto-newline . t)
                    (c-basic-offset . 2)
                    (c-block-comments-indent-p . nil)
                    (c-comment-continuation-stars . "// ")
                    (c-comment-only-line-offset . (0 . 0))
                    (c-default-style . "my-cc-style")
                    (c-electric-pound-behavior . '(alignleft))
                    (c-hanging-comment-ender-p . nil)
                    (c-hanging-comment-starter-p . t)
                    (c-hungry-delete-key . t)
                    (c-recognize-knr-p . nil)
                    (c-tab-always-indent . 'other)
                    (c-cleanup-list . (scope-operator
                                       empty-defun-braces
                                       defun-close-semi
                                       list-close-comma
                                       scope-operator
                                       ))
                    (c-offsets-alist . (
                                        (access-label . -)
                                        (arglist-close . 0)
                                        (case-label . 0)
                                        (cpp-macro . 0)
                                        (cpp-macro-cont . ++)
                                        (extern-lang-open . 0)
                                        (inextern-lang . 0)
                                        (inher-cont . c-lineup-java-inher)
                                        (inline-open . 0)
                                        (innamespace . 0)
                                        (knr-argdecl-intro . +)
                                        (label . 0)
                                        (namespace-open . 0)
                                        (statement-block-intro . +)
                                        (statement-case-intro . +)
                                        (statement-case-open . 0)
                                        (statement-cont . +)
                                        (substatement-open . 0)
                                        (template-args-cont . +)
                                        )) ; end c-offsets-alist
                    (c-recognize-knr-p . nil)
                    )
                  'make-default)        ; end c-add-style, 't' makes this new style default

          ;;
          ;; configure local keymappings
          ;;
          (define-key c-mode-base-map [(meta c)] 'comment-region)
          (define-key c-mode-base-map [(meta s)] 'w32-dev-show-buffer-in-devstudio)
          (define-key c-mode-base-map [(meta \;)] 'c-scope-operator)
          (define-key c-mode-base-map [(meta \\)] 'c-indent-line-or-region)
          (define-key c-mode-base-map [(return)] 'newline-and-indent)
          (define-key c-mode-base-map [(f4)] 'find-file)
;          (define-key c-mode-base-map [f5] 'gud-cont)
          (define-key c-mode-base-map [f5] 'compile-current-buffer)
          (define-key c-mode-base-map [(control f5)] 'mifi-compile)
          (define-key c-mode-base-map [S-f5] 'mifi-debug)
          (define-key c-mode-base-map [f6] (lambda () (interactive) (manual-entry (current-word))))
;          (define-key c-mode-base-map [f6] 'gud-next)
          (define-key c-mode-base-map [S-f6] 'gud-step)
;          (define-key c-mode-base-map [f7] 'gud-down)
;          (define-key c-mode-base-map [S-f7] 'gud-up)
          (define-key c-mode-base-map [f8] 'gud-break)
          (define-key c-mode-base-map [S-f8] 'gud-tbreak)
          (define-key c-mode-base-map [(control f8)] 'gud-remove)
          (define-key c-mode-base-map [(meta down)] '(lambda () (interactive) (scroll-up 1)))
          (define-key c-mode-base-map [(meta up)] '(lambda () (interactive) (scroll-down 1)))
          (define-key c-mode-base-map [(meta \[)] 'c-beginning-of-defun)
          (define-key c-mode-base-map [(meta \])] 'c-end-of-defun)
          (define-key c-mode-base-map [(meta \=)] 'w32-dev-align-equals)
          (define-key c-mode-base-map [(control -)] 'w32-dev-append-hypens-to-fill)
          ;;
          ;; configure local vars for cc-mode based items
          ;;
          ;;

          (add-hook 'c-mode-hook
                    '(lambda () "" (interactive)
                       (setq c-comment-continuation-stars "* ") ; override default comment continuation
                       )
                    )


          (add-hook 'c-mode-common-hook
                    '(lambda () "" (interactive)
                       (c-set-style "my-cc-style")
                       (add-hook 'local-write-file-hooks 'untabify-on-exit)
;                        (make-local-variable 'compile-command)
                       (setq-default compilation-ask-about-save nil
                                     compilation-read-command nil
                                     compilation-scroll-output t
                                     c-auto-newline t
                                     c-electric-pound-behavior '(alignleft)
                                     c-hungry-delete-key t
                                     c-block-comments-indent-p nil
                                     c-tab-always-indent 'other
                                     c-recognize-knr-p nil
                                     c-default-style "my-cc-style"
                                     c-recognize-knr-p nil
                                     )
                       )                ; end lambda
                    )                   ; end add-hook

          ;;--------------------------------------------------------------------------------

          ))

;**************************************************************
;**************************************************************
;;  Makefile MODE
;**************************************************************
;**************************************************************

;;-----------------------------------------------------------
;; post-load hook : makefile-mode

;;
;; makefile mode enhancements
;;

(add-hook 'makefile-mode-hook
          (function
           (lambda ()
             (make-local-variable 'compile-command)
             (setq compile-command
                   (concat "make -f " (buffer-name))))))


(eval-after-load
    "make-mode"
  '(add-hook 'makefile-mode-hook
             '(lambda () "" (interactive)
                (setq compilation-ask-about-save nil)
                (define-key makefile-mode-map [(f5)] 'shell-command)
                (define-key makefile-mode-map [(control f5)] 'compile)
                (define-key makefile-mode-map [(meta c)] 'comment-region)
                (define-key makefile-mode-map [(meta down)] '(lambda () (interactive) (scroll-up 1)))
                (define-key makefile-mode-map [(meta up)] '(lambda () (interactive) (scroll-down 1)))
                )                       ; end lambda
             )                          ; end add-hook
  )                                     ; end eval-after-load

;;


;**************************************************************
;**************************************************************
;;  Fortran MODE
;**************************************************************
;**************************************************************
;;  set personalizations for fortran mode
;;
;;  (add-hook 'fortran-mode-hook (require 'ftnchek-mode "ftnchek"))

(defun insert-fortran-makefile-name (str)
  "Insert the name of the makefile for this budder at its end as a
local variable `makefile-name'."
  (interactive "fFile name: ")
  (save-excursion
    (goto-char (point-max))
    (insert "\nccc Local Variables: \n")
    (insert "ccc mode: Fortran\n")
    (insert (concat "ccc makefile-name: \"" (file-name-nondirectory str) "\""))
    (insert "\nccc End: \n")))

(add-hook 
 'fortran-mode-hook
 '(lambda ()
    (add-hook 'local-write-file-hooks 'untabify-on-exit)
;    (require 'ftnchek-mode "ftnchek")
    ;; following line is all you are asking for.
    (setq fortran-blink-matching-if t)
    ;;
    ;; remaining stuff is all the fancy stuff I prefer ...
    ;;
    (fortran-auto-fill-mode 1)
    (setq abbrev-mode 't)
    (setq fortran-line-number-indent 4)
    (setq comment-column 40)            ; Resets inline comment column.
    (setq fortran-comment-region "c  #")
    (setq fortran-comment-indent-style 'relative)
    (setq fortran-comment-line-extra-indent 0)
    (setq fortran-continuation-indent 6)
    (setq fortran-continuation-string "&")
;    (display-column-mode 1)             ; see "column.el" for column-mode
    (define-key fortran-mode-map [f4] 'insert-fortran-makefile-name)
    (define-key fortran-mode-map [(return)] 'fortran-indent-new-line)
    (define-key fortran-mode-map [f5] 'gud-cont)
    (define-key fortran-mode-map [(control f5)] 'mifi-compile)
    (define-key fortran-mode-map [S-f5] 'mifi-debug)
    (define-key fortran-mode-map [f6] 'gud-next)
    (define-key fortran-mode-map [S-f6] 'gud-step)
;    (define-key fortran-mode-map [f7] 'gud-down)
;    (define-key fortran-mode-map [S-f7] 'gud-up)
    (define-key fortran-mode-map [f8] 'gud-break)
    (define-key fortran-mode-map [S-f8] 'gud-tbreak)
    (define-key fortran-mode-map [(control f8)] 'gud-remove)
    (define-abbrev fortran-mode-abbrev-table ";do" "" 'fortran-skeleton-do-enddo)
    (define-abbrev fortran-mode-abbrev-table ";doi" "" 'fortran-skeleton-do-enddo-i)
    (define-abbrev fortran-mode-abbrev-table ";if" "" 'fortran-skeleton-if-endif)
    (define-abbrev fortran-mode-abbrev-table ";ife" "" 'fortran-skeleton-if-else-endif)
    ))

(define-skeleton fortran-skeleton-do-enddo
  "Insert an do - end do region" "Enter cycle (default: i = 1, n): "
  >"do " str | "i = 1, n" \n
   _ > \n
  "end do" >)

;; (define-skeleton fortran-skeleton-do-enddo
;;   "Insert an do - end do region" nil
;;   >"do "_ \n
;;    -3"end do")

(define-skeleton fortran-skeleton-do-enddo-i
  "Insert an do i= - end do region" nil
  >"do i = 1, "_ \n
   -3"end do")

(define-skeleton fortran-skeleton-if-endif
  "Insert an if - end if region" "Enter condition : "
  >"if (" str ") then"\n
  _ > \n
  "end if">)

;;(define-skeleton fortran-skeleton-if-endif
;;  "Insert an if - end if region" nil
;;  >"if (" _ ") then"\n
;;   -3"end if")

(define-skeleton fortran-skeleton-if-else-endif
  "Insert an if - else - end if region" "Enter condition : "
  >"if (" str ") then"\n
  _ > \n
  "else"> \n
  "end if">)

;;(define-skeleton fortran-skeleton-if-else-endif
;;  "Insert an if - else - end if region" nil
;;  >"if (" _ ") then"\n
;;  -3"else" \n
;;   -3"end if")

(setq skeleton-end-hook nil)


; -----------------------------------------------------------------------------------------------------------------

(defun untabify-on-exit () 
  "untabify the buffer before writing it to disk"
  ;; untabify the buffer
  (untabify  (point-min) (point-max))
  )


; (setq gdb-mode-hook
;       '(lambda ()
;          (define-key gud-mode-map [f6] 'gud-next)
;          (define-key gud-mode-map [S-f6] 'gud-step)
;          (define-key gud-mode-map [f7] 'gud-down)
;          (define-key gud-mode-map [S-f7] 'gud-up)
;          (define-key gud-mode-map [f8] 'gud-break)
;          (define-key gud-mode-map [S-f8] 'gud-tbreak)
;          (define-key gud-mode-map [(control f8)] 'gud-remove)
;          ))


;;----------------------------------------------------------------------------------------

;**************************************************************
;**************************************************************
;;  Gnuplot MODE
;**************************************************************
;**************************************************************
;;  set personalizations for Gnuplot mode

;;--------------------------------------------------------------------
;; Lines enabling gnuplot-mode

;; these lines enable the use of gnuplot mode
  (autoload 'gnuplot-mode "gnuplot" "gnuplot major mode" t)
  (autoload 'gnuplot-make-buffer "gnuplot" "open a buffer in gnuplot mode" t)

;; this line automatically causes all files with the .gp extension to
;; be loaded into gnuplot mode
  (setq auto-mode-alist (append '(("\\.gp$" . gnuplot-mode)) auto-mode-alist))

;; This line binds the function-9 key so that it opens a buffer into
;; gnuplot mode 
  (global-set-key [(f9)] 'gnuplot-make-buffer)

;; end of line for gnuplot-mode
;;--------------------------------------------------------------------

(custom-set-variables
  ;; custom-set-variables was added by Custom -- don't edit or cut/paste it!
  ;; Your init file should contain only one such instance.
 '(bibtex-autokey-name-length 3)
 '(bibtex-autokey-titleword-length 0)
 '(bibtex-autokey-titlewords 0)
 '(bibtex-autokey-titlewords-stretch 0)
 '(bibtex-autokey-year-title-separator "")
 '(cdlatex-math-modify-alist (quote ((120 "\\ve" nil t nil nil))))
 '(dired-listing-switches "-alv")
 '(ps-paper-type (quote a4))
 '(reftex-insert-label-flags (quote ("s" "esft")))
 '(swbuff-exclude-buffer-regexps (quote ("^ " "^\\*.*\\*" "\\.bbdb" "\\.news.*" "diary")))
 '(tooltip-gud-tips-p t))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                   END of SITE-LISP.EL
;;                   START of .EMACS(MIFI)
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;




;(setq default-frame-alist
;      '(
;        (cursor-color . "red")
;       (font . "-*-Courier New-normal-r-*-*-13-97-96-96-c-*-iso8859-2")
;        (foreground-color . "navy")
;        (foreground-color .  "deep pink")
;        (foreground-color .  "black")    
;        (background-color . "linen")     
;        (background-color . "white")
;        (top . 1) (left . 220)
;        (width . 850) (height . 80) 
;        ))





(put 'narrow-to-region 'disabled nil)


;;; Calendar and Diary settings
(setq calendar-latitude 48.09)
(setq calendar-longitude 17.06)
(setq calendar-location-name "Bratislava, SK")
(setq calendar-time-zone 60)
(setq calendar-standard-time-zone-name "Stredna Europa")
(setq cal-tex-which-days '( 1 2 3 4 5 6 0))
(setq calendar-week-start-day 1)
(setq cal-tex-diary t)
(setq european-calendar-style t)
(setq view-diary-entries-initially t)
(setq number-of-diary-entries 7)
(setq appt-issue-message t)
(display-time)
(add-hook 'diary-hook 'appt-make-list) 
(add-hook 'diary-display-hook 'fancy-diary-display)
(add-hook 'list-diary-entries-hook 'sort-diary-entries t)
;(diary)


;;setq dired-guess-shell-alist-user
;;      ;; Try to run xdvi
;;     '(("\\.tex$" (concat "xdvi "
;;                           (file-name-sans-extension (dired-get-filename))
;;                           ".dvi"))))


;;;;;;		Convert Dos ^M End Of Line
;;This should work automagically in Emacs >= v20
;;See also crypt++.el; 'unix2dos' & 'tr' system commands.

;;Approach 1:
;;Unix \n style:
;(setq default-buffer-file-coding-system 'undecided-unix)
;;DOS / Windows / Mac / OS/2 CRLF style:
;(setq default-buffer-file-coding-system 'undecided-dos)

;;Approach 2:	Remove ^M on read, restore it on write
;(require 'dos-mode)

;;Approach 3:	Convert eols by OS
;(load-library "eol-conversion")	;(require 'eol-conversion)?
;(set-buffer-eol-conversion 'dos)

;;Approach 4:	Change EOL conversion, as displayed in modeline:
;(setq eol-mnemonic-dos (string-to-char "\\"))
;(setq eol-mnemonic-unix (string-to-char "/"))
;(setq eol-mnemonic-mac (string-to-char ":"))
;(setq eol-mnemonic-undecided (string-to-char "?"))



; From: Syver Enstad <syver-en+usenet@online.no>
; Subject: Re: compile output in a different window
; 
; "Allan Peda" <luzkstovarna@spammotel.com> writes:
; 
; > It bothers me that when I run a make -k on my source the window spilts
; > to show me the results of the compilation.
; > 
; > How would I relegate all compilation output to an enrirely diffrent
; > window?  

; What I've learned to do is this :
(setq special-display-buffer-names '("*Help*" "*Apropos*" 
				     "*JPython Output*" 
				     "*JPython*"
				     "*Python Output*"
				     "*Python*"
				     "*compilation*"
				     "*shell*"))

;**************************************************************
;**************************************************************
;PHP Mode
;**************************************************************
;**************************************************************
(load-library "php-mode")

;; If you want colorization, turn on global-font-lock or
;; add this to your .emacs:
   (add-hook 'php-mode-user-hook 'turn-on-font-lock)
(add-to-list 'auto-mode-alist '("\\.php\\'" . php-mode))


;**************************************************************
;**************************************************************
;URL Browsing
;**************************************************************
;**************************************************************
 ;:* URL viewing
; (setq browse-url-browser-function 'browse-url-lynx-emacs)
 (setq browse-url-browser-function 'browse-url-netscape)
 (setq browse-url-netscape-program "mozilla")
 ;
 (if (not (eq window-system 'x))
     (setq browse-url-browser-function 'browse-url-w3))
 ;
 (global-set-key "\C-cu." 'browse-url-at-point)
 (global-set-key "\C-cub" 'browse-url-of-buffer)
 (global-set-key "\C-cuf" 'browse-url-of-file)
 ;
 (add-hook 'dired-mode-hook
           (lambda ()
             (local-set-key "\C-cuf" 'browse-url-of-dired-file)))
 ;
 (if (boundp 'browse-url-browser-function)
     (global-set-key "\C-cuu" browse-url-browser-function)
   (eval-after-load
    "browse-url"
    '(global-set-key "\C-cuu" browse-url-browser-function)))
 ;
 (add-hook 'browse-url-of-file-hook 'browse-url-netscape-reload)
 ;
 (setq browse-url-save-file t)
;**************************************************************
;**************************************************************

;;Autoinsert headers
;;From: kgold@watson.ibm.com (kgold)
;;Newsgroups: gnu.emacs.help
;;Subject: Re: auto-insert mode
;;Date: 12 Nov 2001 21:28:22 GMT
;;	(add-hook 'find-file-hooks 'auto-insert)
;;	(setq auto-insert-directory (expand-file-name "~/.autoinsert/"))
;;	(setq auto-insert-query nil)
;;	(define-auto-insert "\\.c\\'"		"autoinsert.c")
;;	(define-auto-insert "\\.cpp\\'"		"autoinsert.c")
;;	(define-auto-insert "\\.java\\'"	"autoinsert.java")
;;	(define-auto-insert "\\.txt\\'"		"autoinsert.txt")


; (setq initial-frame-alist       ; initial frame
;       '(
;         (top    . 100) 
;         (left   . 150)
;         (width  . 80) 
;         (height . 50)
;         (foreground-color       . "antiquewhite1")
;         (background-color       . "navy")
;         (cursor-color           . "red")
;         (mouse-color            . "antiquewhite1")
;         ))

; (setq default-frame-alist       ; subsequent frames
;       '(
;         (top    . 100) 
;         (left   . 150)
;         (width  . 80) 
;         (height . 50)
;         (foreground-color       . "antiquewhite1")
;         (background-color       . "navy")
;         (cursor-color           . "red")
;         (mouse-color            . "antiquewhite1")
;         ))

;(set-default-font
;  (create-fontset-from-ascii-font (cdr (assq 'font (frame-parameters)))))


;; Language and coding system
(setq load-path (cons "/usr/share/emacs/leim/" load-path))
; temporary hack
(load-file "/usr/share/emacs/leim/quail/slovak.el")
(set-language-environment "slovak")
(set-input-method "slovak-prog-2")
;(set-default-font  "-*-fixed-medium-r-normal-*-16-*-*-*-*-*-fontset-standard")
(codepage-setup '1250)
(prefer-coding-system 'cp1250)
;(prefer-coding-system 'iso-8859-1)
(define-coding-system-alias 'windows-1250 'cp1250)
(setq default-input-method "slovak-prog-2")
(set-selection-coding-system 'cp1250)
;; to find the font name type in the Scratch: (w32-select-font) and press ctrl-j just after ')'
(if window-system
    (set-default-font
     (create-fontset-from-ascii-font (cdr (assq 'font (frame-parameters))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;                   start desktop  / MUST be at end
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(desktop-load-default)
(desktop-read)
;; remove desktop-kill from emacs-kill-hook
(remove-hook 'kill-emacs-hook 'desktop-kill)
(setq desktop-enable nil)

