;; From: Stefan Monnier <foo@acm.com> (monnier+gnu.emacs.help/news/@flint.cs.yale.edu)
;; Subject: Re: Making outline mode more useful? 
;; Newsgroups: comp.emacs, gnu.emacs.help
;; Date: 2001-05-17 09:10:29 PST 


;; > 1) To create a new item at the current level easily (e.g., by typing Ret)

(defun outline-new-heading ()
  "Insert a new heading at same depth at point."
  (interactive)
  (let ((head (save-excursion
                (outline-back-to-heading)
                (match-string 0))))
    (unless (or (string-match "[ \t]\\'" head)
                (not (string-match outline-regexp (concat head " "))))
      (setq head (concat head " ")))
    (unless (bolp) (end-of-line) (newline))
    (insert head)
    (unless (eolp)
      (save-excursion (newline)))))

;; > 2) To promote an item easily (e.g., by typing Shift-Tab)

(defun outline-promote ()
  "Promote the current heading higher up the tree."
  (interactive)
  (outline-back-to-heading)
  (let ((new-head (save-excursion
                    (save-match-data
                      (outline-up-heading 1 t)
                      (match-string 0)))))
    (replace-match new-head)))

;; > 3) To demote an item easily (e.g., by typing Tab)

(defun outline-demote ()
  "Demote the current heading lower down the tree."
  (interactive)
  (outline-back-to-heading)
  (let* ((level (funcall outline-level))
	 (new-head (save-excursion
		     (save-match-data
		       (while (and (not (eobp))
				   (progn
				     (outline-next-heading)
				     (<= (funcall outline-level) level))))
		       (when (eobp)
			 ;; Try again from the beginning of the buffer.
			 (while (and (not (eobp))
				     (progn
				       (outline-next-heading)
				       (<= (funcall outline-level) level)))))
		       (unless (eobp) (match-string 0))))))
    (unless new-head
      (save-match-data
	;; Bummer!! There is no lower heading in the buffer.
	;; Let's try to invent one.
	(let ((head (match-string 0)))
	  ;; Try to repeat the first char.
	  (setq new-head (concat (substring head 0 1) head))
	  (if (string-match outline-regexp new-head)
	      ;; Why bother checking that it is indeed of lower level ?
	      nil
	    ;; Didn't work: keep it as is so at least it's still a heading.
	    (setq new-head head)))))
    (replace-match new-head)))

;; Note that the above functions only work 90%.  The reason is that
;; outline-mode only knows how to recognize a heading, not how to create one.
;; So the functions above work by looking around for a heading of the
;; appropriate depth and use that.  But if you have a heading like `***'
;; and try to demote it (i.e. to `****'), it will only work if there is already
;; a heading at that `****' level somewhere.
;; 
;; This problem should not be too serious for the first two functions above,
;; since there usually already is a heading at the same or higher level
;; higher up in the buffer, but for `demote', it's not always the case.
;; 
;; Of course, in the standard outline-mode case where heading have a regular
;; structure like `***', it's easy enough for outline-mode to do the right
;; thing, but when outline-minor-mode is used in LaTeX, outline-minor-mode
;; does not have enough information to easily figure out that demoting
;; `\chapter' means to insert `\section'.
;; 
;; > Of course, text filling should always follow the current indentation level.
;; 
;; I'm not sure what you mean here.
;; 
;; > Has anyone come up with a better outline mode, or with a way to add
;; > the above capabilities?
;; 
;; There are several other outline-like modes.  Emacs comes with hideshow.el
;; (more targeted at programming language than actual outlines) and allout.el,
;; there's also folding.el and probably others,