;;; tex-site.el - Site specific variables.  Don't edit.

;; Copyright (C) 1991, 2000, 2001 Kresten Krab Thorup
;; Copyright (C) 1993, 1994, 1997, 1999,
;;               2003, 2004 Free Software Foundation, Inc.

;; Author: Per Abrahamsen <abraham@dina.kvl.dk>
;; Maintainer: auc-tex@sunsite.dk
;; Keywords: tex

;; This file is part of AUCTeX.

;; AUCTeX is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; AUCTeX is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with AUCTeX; see the file COPYING.  If not, write to the Free
;; Software Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
;; 02111-1307, USA.

;;; Commentary:

;; This file used to contain variables customized for the local site.
;; This way of customizing AUCTeX should no longer be used.  All
;; relevant variables can be customized by using
;; M-x customize-variable RET
;; on them.  Copying stuff into this file, as was recommended
;; previously, would interfere with the operation of customize.
;;
;; Basically, this file contains a bunch of autoloads and
;; automatically generated information.  AUCTeX is enabled by placing
;; (require 'tex-site)
;; into a user's .emacs file, or the administrator can insert it in
;; the site-start.el file for a site-wide default.
;;
;; The right place for this file is in the `site-lisp' directory.

;;; Code:

(if (< emacs-major-version 21)
  (error "AUCTeX requires Emacs 21 or later"))

(defvar no-doc
  "This function is part of AUCTeX, but has not yet been loaded.
Full documentation will be available after autoloading the function."
  "Documentation for autoload functions.")

(defvar TeX-lisp-directory "/home/fikar/emacs/site-lisp/auctex"
  "*The directory where the AUCTeX lisp files are located.")

(defcustom TeX-auto-global "/usr/local/var/auctex"
  "*Directory containing automatically generated information.
Must end with a directory separator.

For storing automatic extracted information about the TeX macros
shared by all users of a site."
  :group 'TeX-file
  :type 'directory)

(defcustom TeX-macro-global '("/usr/share/texmf/tex/" "/usr/share/texmf/bibtex/bst/")
  "Directories containing the sites TeX macro files and style files.
The directory names *must* end with a slash."
  :group 'TeX-file
  :type '(repeat (directory :format "%v")))

;;; Autoloads:

(add-to-list 'auto-mode-alist '("\\.dtx\\'" . doctex-mode))
(add-to-list 'auto-mode-alist '("\\.drv\\'" . latex-mode))

(add-to-list 'load-path TeX-lisp-directory)

;; This hook will store bibitems when you save a BibTeX buffer.
(add-hook 'bibtex-mode-hook 'BibTeX-auto-store)
(autoload 'BibTeX-auto-store "latex" no-doc t)

(autoload 'tex-mode "tex" no-doc t)
(autoload 'plain-tex-mode "tex" no-doc t)
(autoload 'ams-tex-mode "tex" no-doc t)
(autoload 'context-mode "context" no-doc t)
(autoload 'doctex-mode "latex" no-doc t)
(autoload 'TeX-auto-generate "tex" no-doc t)
(autoload 'TeX-auto-generate-global "tex" no-doc t)
(autoload 'TeX-insert-quote "tex" no-doc t)
(autoload 'TeX-submit-bug-report "tex" no-doc t)
(autoload 'japanese-plain-tex-mode "tex-jp" no-doc t)
(autoload 'japanese-latex-mode "tex-jp" no-doc t)
(autoload 'japanese-slitex-mode "tex-jp" no-doc t)
(autoload 'texinfo-mode "tex-info" no-doc t)
(autoload 'latex-mode "latex" no-doc t)

(defcustom TeX-command-list
  ;; Changed to double quotes for Windows afflicted people.
  `(("TeX" "%(PDF)%(tex) %S%(PDFout) \"%(mode)\\input %t\""
     TeX-run-TeX nil
     (plain-tex-mode ams-tex-mode texinfo-mode) :help "Run plain TeX")
    ("LaTeX" "%l \"%(mode)\\input{%t}\""
     TeX-run-TeX nil
     (latex-mode doctex-mode) :help "Run LaTeX")
    ("CsLaTeX" "cslatexw %t"
     TeX-run-TeX nil
     (latex-mode doctex-mode) :help "Run CsLaTeX")
    ("PdfCsLaTeX" "pdfcslatexw %t"
     TeX-run-TeX nil
     (latex-mode doctex-mode) :help "Run PdfCsLaTeX")
    ("View" "xdvi -expert -s 5 %s"      TeX-run-discard nil nil :help "View DVI File")
    ("ViewPS" "gv %f"        TeX-run-silent nil nil :help "View PS File")
    ("ViewPDF" "acroread %s.pdf" TeX-run-silent nil nil :help "View PDF File")
    ("Ps2Pdf" "ps2pdf %f" TeX-run-command nil nil :help "Convert PS to PDF")
	;; Not part of standard TeX.
    ("Makeinfo" "makeinfo %t" TeX-run-compile nil
     (texinfo-mode) :help "Run Makeinfo with Info output")
    ("Makeinfo HTML" "makeinfo --html %t" TeX-run-compile nil
     (texinfo-mode) :help "Run Makeinfo with HTML output")
    ("AmSTeX" "%(PDF)amstex %S%(PDFout) \"%(mode)\\input %t\""
     TeX-run-TeX nil (ams-tex-mode) :help "Run AMSTeX")
    ;; support for ConTeXt  --pg
    ;; first version of ConTeXt to support nonstopmode: 2003.2.10
    ("ConTeXt" "texexec --once --texutil %(execopts)%t"
     TeX-run-TeX nil (context-mode) :help "Run ConTeXt once")
    ("ConTeXt Full" "texexec %(execopts)%t"
     TeX-run-TeX nil
     (context-mode) :help "Run ConTeXt until completion")
    ;; --purge %s does not work on unix systems with current texutil
    ;; check again october 2003 --pg
    ("ConTeXt Clean" "texutil --purgeall" TeX-run-interactive nil
     (context-mode) :help "Clean temporary ConTeXt files")
    ("BibTeX" "bibtex %s" TeX-run-BibTeX nil t :help "Run BibTeX")
;    ,(if (or window-system (getenv "DISPLAY"))
;	'("View" "%V" TeX-run-discard t t :help "Run Viewer")
;       '("View" "dvi2tty -q -w 132 %s" TeX-run-command t t
;	 :help "Run Text viewer"))
    ("Print" "%p" TeX-run-command t t :help "Print the file")
    ("Queue" "%q" TeX-run-background nil t :help "View the printer queue"
     :visible TeX-queue-command)
    ("File" "%(o?)dvips %d -o %f " TeX-run-command nil nil :help "Generate PostScript file")
    ("Index" "makeindex %s" TeX-run-command nil t :help "Create index file")
    ;; (list "Check" "chktex -v3 %s" TeX-run-compile nil t :help "Check )
    ;; Uncomment the above line and comment out the next line to
    ;; use `chktex' instead of `lacheck'.
    ("Check" "lacheck %s" TeX-run-compile nil (latex-mode)
     :help "Check LaTeX file for correctness")
    ("Spell" "<ignored>" TeX-run-ispell-on-document nil t
     :help "Spell-check the document")
    ("Other" "" TeX-run-command t t :help "Run an arbitrary command"))
  "List of commands to execute on the current document.

Each element is a list, whose first element is the name of the command
as it will be presented to the user.

The second element is the string handed to the shell after being
expanded.  The expansion is done using the information found in
`TeX-expand-list'.

The third element is the function which actually start the process.
Several such hooks has been defined:

TeX-run-command: Start up the process and show the output in a
separate buffer.  Check that there is not two commands running for the
same file.  Return the process object.

TeX-run-format: As `TeX-run-command', but assume the output is created
by a TeX macro package.  Return the process object.

TeX-run-TeX: For TeX output.

TeX-run-interactive: Run TeX or LaTeX interactively.

TeX-run-BibTeX: For BibTeX output.

TeX-run-compile: Use `compile' to run the process.

TeX-run-shell: Use `shell-command' to run the process.

TeX-run-discard: Start the process in the background, discarding its
output.

TeX-run-background: Start the process in the background, show output
in other window.

TeX-run-silent: Start the process in the background.

TeX-run-dviout: Special hook for the Japanese dviout previewer for
PC-9801.

To create your own hook, define a function taking three arguments: The
name of the command, the command string, and the name of the file to
process.  It might be useful to use `TeX-run-command' in order to
create an asynchronous process.

If the fourth element is non-nil, the user will get a chance to
modify the expanded string.

The fifth element indicates in which mode(s) the command should be
present in the Command menu.  Use t if it should be active in any
mode.  If it should only be present in some modes, specify a list with
the respective mode names.

Any additional elements get just transferred to the respective menu entries.
"
  :group 'TeX-command
  :type '(repeat (group :value ("" "" TeX-run-command nil t)
			(string :tag "Name")
			(string :tag "Command")
			(choice :tag "How"
				:value TeX-run-command
				(function-item TeX-run-command)
				(function-item TeX-run-format)
				(function-item TeX-run-TeX)
				;; leave the following line in
				;; customization? Replaced (but still
				;; available) with TeX-run-TeX --pg
				(function-item TeX-run-LaTeX)
				(function-item TeX-run-interactive)
				(function-item TeX-run-BibTeX)
				(function-item TeX-run-compile)
				(function-item TeX-run-shell)
				(function-item TeX-run-discard)
				(function-item TeX-run-background)
				(function-item TeX-run-silent)
				(function-item TeX-run-dviout)
				(function :tag "Other"))
			(boolean :tag "Prompt")
			(choice :tag "Modes"
				(const :tag "All" t)
				(set (const :tag "Plain TeX" plain-tex-mode)
				     (const :tag "LaTeX" latex-mode)
				     (const :tag "DocTeX" doctex-mode)
				     (const :tag "ConTeXt" context-mode)
				     (const :tag "Texinfo" texinfo-mode)
				     (const :tag "AmSTeX" ams-tex-mode)))
			(repeat :tag "Menu elements" :inline t sexp))))


(provide 'tex-site)

;;; tex-site.el ends here
