;; textilde.el -- Code to insert tilde '~' after certain words in LaTeX mode
;; Copyright (c) 2001 Miroslav Fikar

;; textilde.el, version: 0.02 date: 2001/10/07 

;; Author: Miroslav Fikar <fikar@cvt.stuba.sk>
;; Keywords: tex
;;
;; New versions: mailto:fikar@cvt.stuba.sk
;;
;; This file is not part of GNU Emacs
;;
;; COPYRIGHT NOTICE
;;
;; This program is free software you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.
;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;
;;; Commentary:
;;
;;  This code inserts ~ after words defined below. If distinguishes
;;  between text and math mode. 
;;
;;  To install, put this file on your load-path and compile it, and
;;  add to your .emacs:
;;
;;     (require 'textilde)
;;
;;  To turn it on/off use M-x textilde-toogle RET
;;  Other functions
;;     M-x textilde-off
;;     M-x textilde-on
;;
;;  As it is my first lisp program, it is very badly written. But it
;;  works (sometimes).  It would be nice to have it as a minor mode
;;  with indicator and to be able to add new words for example in
;;  .emacs, or in a customisation buffer.
;;--------------------------------------------------------------------------
;;
;;  BUGS:
;;
;;  Tell me
;;--------------------------------------------------------------------------
;; 0.02 : added do-auto-fill if ordinary space is inserted

;;; Code:

(require 'texmathp)
(eval-after-load
    "tex-site"
  '(add-hook 'LaTeX-mode-hook
             '(lambda () "" (interactive)
                (define-key LaTeX-mode-map [?\ ] 'textilde)
              )             ; end lambda
   )                        ; end add-hook
)                           ; end eval-after-load


(defvar textilde-list (list "A" "I" "i" "K" "k" "O" "o" "S" "s" "U" "u"
"V" "v" "Z" "z" "Obr." "obr." "Tab." "tab.")
  "List of words after those should be tilde.")

(defvar textilde-is-on t
  "Initially, textilde-mode is on")

(defun textilde-toogle ()
  "Toogle if tilde is on or off."
  (interactive)
  (if textilde-is-on
      (textilde-off)
    (textilde-on)))

(defun textilde-off ()
  "Tilde will be off"
  (interactive)
  (setq textilde-is-on nil)
  (define-key LaTeX-mode-map [?\ ] 'self-insert-command))

(defun textilde-on ()
  "Tilde will be on"
  (interactive)
  (setq textilde-is-on t)
  (define-key LaTeX-mode-map [?\ ] 'textilde))

(defun word-in-textilde-list-p (str liststr)
  "See if STR is in LISTSTR (t-nil)"
  (let ((result nil))
    (while liststr
      (if (string= (car liststr) str)
          (setq result t))
      (setq liststr (cdr liststr)))
     result))


(defun word-before-point-p ()
  "Return true if a word is before point."
  (save-excursion
  (/= 0 (skip-chars-backward "a-zA-Z."))))

(defun word-before-point ()
  "Return word before point."
  (buffer-substring-no-properties 
   (save-excursion
     (forward-word -1)
     (point)) 
   (point))
)

(defun insert-space ()
  "Insert space and change back to textilde mode."
  (define-key LaTeX-mode-map [?\ ] 'self-insert-command)
;;  (fill-paragraph nil) ;;slows down heavily
  (do-auto-fill)
  (insert " ")
  (define-key LaTeX-mode-map [?\ ] 'textilde)
)
  

(defun textilde ()
  "Insert either tilde (if after certain words in text mode - see
variable `textilde-list'), or ordinary space."
  (interactive)
  (if (and 
       (not (texmathp))
       (word-before-point-p) 
       (word-in-textilde-list-p (word-before-point) textilde-list) )
      (insert "~")
    (insert-space))
)

 

(provide 'textilde)
